/*******************************************************************************
* File Name: CAN_Rx.c  
* Version 1.90
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "CAN_Rx.h"


/*******************************************************************************
* Function Name: CAN_Rx_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void CAN_Rx_Write(uint8 value) 
{
    uint8 staticBits = (CAN_Rx_DR & (uint8)(~CAN_Rx_MASK));
    CAN_Rx_DR = staticBits | ((uint8)(value << CAN_Rx_SHIFT) & CAN_Rx_MASK);
}


/*******************************************************************************
* Function Name: CAN_Rx_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to this drive mode.
*
* Return: 
*  None
*
*******************************************************************************/
void CAN_Rx_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(CAN_Rx_0, mode);
}


/*******************************************************************************
* Function Name: CAN_Rx_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro CAN_Rx_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 CAN_Rx_Read(void) 
{
    return (CAN_Rx_PS & CAN_Rx_MASK) >> CAN_Rx_SHIFT;
}


/*******************************************************************************
* Function Name: CAN_Rx_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 CAN_Rx_ReadDataReg(void) 
{
    return (CAN_Rx_DR & CAN_Rx_MASK) >> CAN_Rx_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(CAN_Rx_INTSTAT) 

    /*******************************************************************************
    * Function Name: CAN_Rx_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 CAN_Rx_ClearInterrupt(void) 
    {
        return (CAN_Rx_INTSTAT & CAN_Rx_MASK) >> CAN_Rx_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
