/*******************************************************************************
* File Name: CANx.h
* Version 2.30
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the CAN Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_CAN_CANx_H)
#define CY_CAN_CANx_H

#include "cyfitter.h"
#include "cytypes.h"
#include "CyLib.h"

/* Check to see if required defines such as CY_PSOC5LP are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5LP)
    #error Component CAN_v2_30 requires cy_boot v3.0 or later
#endif /* (CY_PSOC5LP) */

extern uint8 CANx_initVar;

#define CANx_INT_ISR_DISABLE        (0u)


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define CANx_ARB_LOST               (0u)
#define CANx_OVERLOAD               (0u)
#define CANx_BIT_ERR                (0u)
#define CANx_STUFF_ERR              (0u)
#define CANx_ACK_ERR                (0u)
#define CANx_FORM_ERR               (0u)
#define CANx_CRC_ERR                (0u)
#define CANx_BUS_OFF                (1u)
#define CANx_RX_MSG_LOST            (0u)
#define CANx_TX_MESSAGE             (0u)
#define CANx_RX_MESSAGE             (1u)
#define CANx_ARB_LOST_USE_HELPER    (1u)
#define CANx_OVERLOAD_USE_HELPER    (1u)
#define CANx_BIT_ERR_USE_HELPER     (1u)
#define CANx_STUFF_ERR_USE_HELPER   (1u)
#define CANx_ACK_ERR_USE_HELPER     (1u)
#define CANx_FORM_ERR_USE_HELPER    (1u)
#define CANx_CRC_ERR_USE_HELPER     (1u)
#define CANx_BUS_OFF_USE_HELPER     (1u)
#define CANx_RX_MSG_LOST_USE_HELPER (1u)
#define CANx_TX_MESSAGE_USE_HELPER  (1u)
#define CANx_RX_MESSAGE_USE_HELPER  (1u)
#define CANx_ADVANCED_INTERRUPT_CFG (0u)

/* TX/RX Function Enable */
#define CANx_TX0_FUNC_ENABLE 1u
#define CANx_TX1_FUNC_ENABLE 1u
#define CANx_TX2_FUNC_ENABLE 1u
#define CANx_TX3_FUNC_ENABLE 0u
#define CANx_TX4_FUNC_ENABLE 0u
#define CANx_TX5_FUNC_ENABLE 0u
#define CANx_TX6_FUNC_ENABLE 0u
#define CANx_TX7_FUNC_ENABLE 0u
#define CANx_RX0_FUNC_ENABLE 1u
#define CANx_RX1_FUNC_ENABLE 1u
#define CANx_RX2_FUNC_ENABLE 1u
#define CANx_RX3_FUNC_ENABLE 0u
#define CANx_RX4_FUNC_ENABLE 0u
#define CANx_RX5_FUNC_ENABLE 0u
#define CANx_RX6_FUNC_ENABLE 0u
#define CANx_RX7_FUNC_ENABLE 0u
#define CANx_RX8_FUNC_ENABLE 0u
#define CANx_RX9_FUNC_ENABLE 0u
#define CANx_RX10_FUNC_ENABLE 0u
#define CANx_RX11_FUNC_ENABLE 0u
#define CANx_RX12_FUNC_ENABLE 0u
#define CANx_RX13_FUNC_ENABLE 0u
#define CANx_RX14_FUNC_ENABLE 0u
#define CANx_RX15_FUNC_ENABLE 0u
#define CANx_RX_MAILBOX_TYPE	0x7u
#define CANx_TX_MAILBOX_TYPE	0x7u


/***************************************
*        Data Struct Definition
***************************************/

/* Stuct for DATA of BASIC CAN mailbox */
typedef struct
{
    uint8 byte[8u];
} CANx_DATA_BYTES_MSG;

/* Stuct for DATA of CAN RX register */
typedef struct
{
    reg8 byte[8u];
} CANx_DATA_BYTES;

/* Stuct for 32-bit CAN register */
typedef struct
{
    reg8 byte[4u];
} CANx_REG_32;

/* Stuct for BASIC CAN mailbox to send messages */
typedef struct
{
    uint32 id;
    uint8 rtr;
    uint8 ide;
    uint8 dlc;
    uint8 irq;
    CANx_DATA_BYTES_MSG *msg;
} CANx_TX_MSG;

/* Constant configutaion of CAN RX */
typedef struct
{
    uint8 rxmailbox;
    uint32 rxcmd;
    uint32 rxamr;
    uint32 rxacr;
} CANx_RX_CFG;

/* Constant configutaion of CAN TX */
typedef struct
{
    uint8 txmailbox;
    uint32 txcmd;
    uint32 txid;
} CANx_TX_CFG;

/* CAN RX registers */
typedef struct
{
    CANx_REG_32 rxcmd;
    CANx_REG_32 rxid;
    CANx_DATA_BYTES rxdata;
    CANx_REG_32 rxamr;
    CANx_REG_32 rxacr;
    CANx_REG_32 rxamrd;
    CANx_REG_32 rxacrd;
} CANx_RX_STRUCT;

/* CAN TX registers */
typedef struct
{
    CANx_REG_32 txcmd;
    CANx_REG_32 txid;
    CANx_DATA_BYTES txdata;
} CANx_TX_STRUCT;

/* Sleep Mode API Support */
typedef struct
{
    uint8  enableState;
    uint32 intSr;
    uint32 intEn;
    uint32 cmd;
    uint32 cfg;    
} CANx_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

uint8 CANx_Init(void) ;
uint8 CANx_Start(void) ;
uint8 CANx_Stop(void) ;
uint8 CANx_Enable(void) ;
uint8 CANx_GlobalIntEnable(void) ;
uint8 CANx_GlobalIntDisable(void) ;
uint8 CANx_SetPreScaler(uint16 bitrate) ;
uint8 CANx_SetArbiter(uint8 arbiter) ;
uint8 CANx_SetTsegSample(uint8 cfgTseg1, uint8 cfgTseg2, uint8 sjw, uint8 sm)
                                                ;
uint8 CANx_SetRestartType(uint8 reset) ;
uint8 CANx_SetEdgeMode(uint8 edge) ;
uint8 CANx_SetOpMode(uint8 opMode) ;
uint8 CANx_RXRegisterInit(reg32 *regAddr, uint32 config)
                                            ;
uint8 CANx_SetIrqMask(uint16 mask) ;
uint8 CANx_GetTXErrorFlag(void) ;
uint8 CANx_GetRXErrorFlag(void) ;
uint8 CANx_GetTXErrorCount(void) ;
uint8 CANx_GetRXErrorCount(void) ;
uint8 CANx_GetErrorState(void) ;
void  CANx_Sleep(void) ;
void  CANx_Wakeup(void) ;
void  CANx_SaveConfig(void) ;
void  CANx_RestoreConfig(void) ;

#if (CANx_ARB_LOST)
    void CANx_ArbLostIsr(void) ;
#endif /* CANx_ARB_LOST */
#if (CANx_OVERLOAD)
    void CANx_OvrLdErrorIsr(void) ;
#endif /* CANx_OVERLOAD */
#if (CANx_BIT_ERR)
    void CANx_BitErrorIsr(void) ;
#endif /* CANx_BIT_ERR */
#if (CANx_STUFF_ERR)
    void CANx_BitStuffErrorIsr(void) ;
#endif /* CANx_STUFF_ERR */
#if (CANx_ACK_ERR)
    void CANx_AckErrorIsr(void) ;
#endif /* CANx_ACK_ERR */
#if (CANx_FORM_ERR)
    void CANx_MsgErrorIsr(void) ;
#endif /* CANx_FORM_ERR */
#if (CANx_CRC_ERR)
    void CANx_CrcErrorIsr(void) ;
#endif /* CANx_CRC_ERR */
#if (CANx_BUS_OFF)
    void CANx_BusOffIsr(void) ;
#endif /* CANx_BUS_OFF */
#if (CANx_RX_MSG_LOST)
    void CANx_MsgLostIsr(void) ;
#endif /* CANx_RX_MSG_LOST */
#if (CANx_TX_MESSAGE)
   void CANx_MsgTXIsr(void) ;
#endif /* CANx_TX_MESSAGE */
#if (CANx_RX_MESSAGE)
    void CANx_MsgRXIsr(void) ;
#endif /* CANx_RX_MESSAGE */

uint8 CANx_RxBufConfig(const CANx_RX_CFG *rxConfig) \
                                   ;
uint8 CANx_TxBufConfig(const CANx_TX_CFG *txConfig) \
                                   ;
uint8 CANx_SendMsg(const CANx_TX_MSG *message) ;
void  CANx_TxCancel(uint8 bufferId) ;
void  CANx_ReceiveMsg(uint8 rxMailbox) ;

#if (CANx_TX0_FUNC_ENABLE)
    uint8 CANx_SendMsg0(void) ;
#endif /* CANx_TX0_FUNC_ENABLE */
#if (CANx_TX1_FUNC_ENABLE)
    uint8 CANx_SendMsg1(void) ;
#endif /* CANx_TX1_FUNC_ENABLE */
#if (CANx_TX2_FUNC_ENABLE)
    uint8 CANx_SendMsgBoot_response(void) ;
#endif /* CANx_TX2_FUNC_ENABLE */
#if (CANx_TX3_FUNC_ENABLE)
    uint8 CANx_SendMsg3(void) ;
#endif /* CANx_TX3_FUNC_ENABLE */
#if (CANx_TX4_FUNC_ENABLE)
    uint8 CANx_SendMsg4(void) ;
#endif /* CANx_TX4_FUNC_ENABLE */
#if (CANx_TX5_FUNC_ENABLE)
    uint8 CANx_SendMsg5(void) ;
#endif /* CANx_TX5_FUNC_ENABLE */
#if (CANx_TX6_FUNC_ENABLE)
    uint8 CANx_SendMsg6(void) ;
#endif /* CANx_TX6_FUNC_ENABLE */
#if (CANx_TX7_FUNC_ENABLE)
    uint8 CANx_SendMsg7(void) ;
#endif /* CANx_TX7_FUNC_ENABLE */
#if (CANx_RX0_FUNC_ENABLE)
    void CANx_ReceiveMsg0(void) ;
#endif /* CANx_RX0_FUNC_ENABLE */
#if (CANx_RX1_FUNC_ENABLE)
    void CANx_ReceiveMsg1(void) ;
#endif /* CANx_RX1_FUNC_ENABLE */
#if (CANx_RX2_FUNC_ENABLE)
    void CANx_ReceiveMsgBoot_cmd(void) ;
#endif /* CANx_RX2_FUNC_ENABLE */
#if (CANx_RX3_FUNC_ENABLE)
    void CANx_ReceiveMsg3(void) ;
#endif /* CANx_RX3_FUNC_ENABLE */
#if (CANx_RX4_FUNC_ENABLE)
    void CANx_ReceiveMsg4(void) ;
#endif /* CANx_RX4_FUNC_ENABLE */
#if (CANx_RX5_FUNC_ENABLE)
    void CANx_ReceiveMsg5(void) ;
#endif /* CANx_RX5_FUNC_ENABLE */
#if (CANx_RX6_FUNC_ENABLE)
    void CANx_ReceiveMsg6(void) ;
#endif /* CANx_RX6_FUNC_ENABLE */
#if (CANx_RX7_FUNC_ENABLE)
    void CANx_ReceiveMsg7(void) ;
#endif /* CANx_RX7_FUNC_ENABLE */
#if (CANx_RX8_FUNC_ENABLE)
    void CANx_ReceiveMsg8(void) ;
#endif /* CANx_RX8_FUNC_ENABLE */
#if (CANx_RX9_FUNC_ENABLE)
    void CANx_ReceiveMsg9(void) ;
#endif /* CANx_RX9_FUNC_ENABLE */
#if (CANx_RX10_FUNC_ENABLE)
    void CANx_ReceiveMsg10(void) ;
#endif /* CANx_RX10_FUNC_ENABLE */
#if (CANx_RX11_FUNC_ENABLE)
    void CANx_ReceiveMsg11(void) ;
#endif /* CANx_RX11_FUNC_ENABLE */
#if (CANx_RX12_FUNC_ENABLE)
    void CANx_ReceiveMsg12(void) ;
#endif /* CANx_RX12_FUNC_ENABLE */
#if (CANx_RX13_FUNC_ENABLE)
    void CANx_ReceiveMsg13(void) ;
#endif /* CANx_RX13_FUNC_ENABLE */
#if (CANx_RX14_FUNC_ENABLE)
    void CANx_ReceiveMsg14(void) ;
#endif /* CANx_RX14_FUNC_ENABLE */
#if (CANx_RX15_FUNC_ENABLE)
    void CANx_ReceiveMsg15(void) ;
#endif /* CANx_RX15_FUNC_ENABLE */

#if(!CANx_INT_ISR_DISABLE)
    /* Interrupt handler */
    CY_ISR_PROTO(CANx_ISR);
#endif /* End CANx_INT_ISR_DISABLE */


/***************************************
*           API Constants
***************************************/

#if (!CANx_INT_ISR_DISABLE)
    /* Number of the CANx_isr interrupt */
    #define CANx_ISR_NUMBER             ((uint8) CANx_isr__INTC_NUMBER)
    /* Priority of the CANx_isr interrupt */
    #define CANx_ISR_PRIORITY           ((uint8) CANx_isr__INTC_PRIOR_NUM)
#endif /* End CANx_INT_ISR_DISABLE */

/* One or more parameters to the function were invalid. */
#define CANx_FAIL                       (0x01u)
#define CANx_OUT_OF_RANGE               (0x02u)

/* PM_ACT_CFG (Active Power Mode CFG Register) */
#define CANx_ACT_PWR_EN                 (CANx_CanIP__PM_ACT_MSK)    /* Power enable mask */

/* PM_STBY_CFG (Alternate Active (Standby) Power Mode CFG Register) */
#define CANx_STBY_PWR_EN                (CANx_CanIP__PM_STBY_MSK)   /* Power enable mask */

/* Number of TX and RX mailboxes */
#define CANx_NUMBER_OF_TX_MAILBOXES     (8u)
#define CANx_NUMBER_OF_RX_MAILBOXES     (16u) 

/* Error status of CAN */
#define CANx_ERROR_ACTIVE               (0x00u)
#define CANx_ERROR_PASIVE               (0x01u)
#define CANx_ERROR_BUS_OFF              (0x10u)


/***************************************
*    Initial Parameter Constants
***************************************/

/* General */
#define CANx_BITRATE             (0u)
#define CANx_CFG_REG_TSEG1       (13u - 1u)
#define CANx_CFG_REG_TSEG2       (2u - 1u)
#define CANx_CFG_REG_SJW         (1u - 1u)
#define CANx_SAMPLING_MODE       (0u)

#define CANx_ARBITER             (0u)
#define CANx_RESET_TYPE          (0u)
#define CANx_SYNC_EDGE           (0u)

/* Interrupts */
#define CANx_INT_ENABLE          (1u)
#define CANx_INIT_INTERRUPT_MASK (((uint16) CANx_INT_ENABLE) | \
                    ((uint16) ((uint16) CANx_ARB_LOST    <<  CANx_ARBITRATION_LOST_SHIFT)) | \
                    ((uint16) ((uint16) CANx_OVERLOAD    <<  CANx_OVERLOAD_ERROR_SHIFT))   | \
                    ((uint16) ((uint16) CANx_BIT_ERR     <<  CANx_BIT_ERROR_SHIFT))        | \
                    ((uint16) ((uint16) CANx_STUFF_ERR   <<  CANx_STUFF_ERROR_SHIFT))      | \
                    ((uint16) ((uint16) CANx_ACK_ERR     <<  CANx_ACK_ERROR_SHIFT))        | \
                    ((uint16) ((uint16) CANx_FORM_ERR    <<  CANx_FORM_ERROR_SHIFT))       | \
                    ((uint16) ((uint16) CANx_CRC_ERR     << (CANx_ONE_BYTE_OFFSET          + \
                                                               CANx_CRC_ERROR_SHIFT)))                 | \
                    ((uint16) ((uint16) CANx_BUS_OFF     << (CANx_ONE_BYTE_OFFSET          + \
                                                               CANx_BUS_OFF_SHIFT)))                   | \
                    ((uint16) ((uint16) CANx_RX_MSG_LOST << (CANx_ONE_BYTE_OFFSET          + \
                                                               CANx_RX_MSG_LOST_SHIFT)))               | \
                    ((uint16) ((uint16) CANx_TX_MESSAGE  << (CANx_ONE_BYTE_OFFSET          + \
                                                               CANx_TX_MESSAGE_SHIFT)))                | \
                    ((uint16) ((uint16) CANx_RX_MESSAGE  << (CANx_ONE_BYTE_OFFSET          + \
                                                                           CANx_RX_MESSAGE_SHIFT))))


/***************************************
*             Registers
***************************************/

#define CANx_TX          ( (volatile CANx_TX_STRUCT XDATA *) CANx_CanIP__TX0_CMD)
#define CANx_RX          ( (volatile CANx_RX_STRUCT XDATA *) CANx_CanIP__RX0_CMD)
#define CANx_INT_SR_REG  (*(volatile CANx_REG_32 XDATA *) CANx_CanIP__CSR_INT_SR)
#define CANx_INT_SR_PTR  ( (reg32 *) CANx_CanIP__CSR_INT_SR)
#define CANx_INT_EN_REG  (*(volatile CANx_REG_32 XDATA *) CANx_CanIP__CSR_INT_EN)
#define CANx_INT_EN_PTR  ( (reg32 *) CANx_CanIP__CSR_INT_EN)
#define CANx_BUF_SR_REG  (*(volatile CANx_REG_32 XDATA *) CANx_CanIP__CSR_BUF_SR)
#define CANx_BUF_SR_PTR  ( (reg32 *) CANx_CanIP__CSR_BUF_SR)
#define CANx_ERR_SR_REG  (*(volatile CANx_REG_32 XDATA *) CANx_CanIP__CSR_ERR_SR)
#define CANx_ERR_SR_PTR  ( (reg32 *) CANx_CanIP__CSR_ERR_SR)
#define CANx_CMD_REG     (*(volatile CANx_REG_32 XDATA *) CANx_CanIP__CSR_CMD)
#define CANx_CMD_PTR     ( (reg32 *) CANx_CanIP__CSR_CMD)
#define CANx_CFG_REG     (*(volatile CANx_REG_32 XDATA *) CANx_CanIP__CSR_CFG)
#define CANx_CFG_PTR     ( (reg32 *) CANx_CanIP__CSR_CFG)
#define CANx_PM_ACT_CFG_REG  (*(reg8 *) CANx_CanIP__PM_ACT_CFG)    /* Power manager */
#define CANx_PM_ACT_CFG_PTR  ( (reg8 *) CANx_CanIP__PM_ACT_CFG)    /* Power manager */
#define CANx_PM_STBY_CFG_REG (*(reg8 *) CANx_CanIP__PM_STBY_CFG)   /* Power manager */
#define CANx_PM_STBY_CFG_PTR ( (reg8 *) CANx_CanIP__PM_STBY_CFG)   /* Power manager */

#define CANx_RX_FIRST_REGISTER_PTR    ((reg32 *) CANx_CanIP__RX0_CMD)
#define CANx_RX_LAST_REGISTER_PTR     ((reg32 *) CANx_CanIP__RX15_ACRD)


/***************************************
*        Register Constants
***************************************/

/* Operation mode */
#define CANx_ACTIVE_MODE              (0x00u)
#define CANx_LISTEN_ONLY              (0x01u)

/* Run/Stop mode */
#define CANx_MODE_STOP                (0x00u)
#define CANx_MODE_START               (0x01u)

/* Transmit buffer arbiter */
#define CANx_ROUND_ROBIN              (0x00u)
#define CANx_FIXED_PRIORITY           (0x01u)

/* Restart type */
#define CANx_MANUAL_RESTART           (0x00u)
#define CANx_AUTO_RESTART             (0x01u)

/* Sampling mode */
#define CANx_ONE_SAMPLE_POINT         (0x00u)
#define CANx_THREE_SAMPLE_POINTS      (0x01u)

/* Edge mode */
#define CANx_EDGE_R_TO_D              (0x00u)
#define CANx_BOTH_EDGES               (0x01u)

/* Extended identifier */
#define CANx_STANDARD_MESSAGE         (0x00u)
#define CANx_EXTENDED_MESSAGE         (0x01u)

/* Write Protect Mask for Basic and Full mailboxes */
#define CANx_TX_WPN_SET             (((uint32) ((uint32) 0x00000001u << CANx_TX_WPNL_SHIFT)) | \
                                                ((uint32) ((uint32) 0x00000001u << (CANx_TWO_BYTE_OFFSET + \
                                                 CANx_TX_WPNH_SHIFT))))

#define CANx_TX_WPN_CLEAR             ((uint32) (~CANx_TX_WPN_SET))

#define CANx_RX_WPN_SET             (((uint32) ((uint32) 0x00000001u << CANx_RX_WPNL_SHIFT)) | \
                                                ((uint32) ((uint32) 0x00000001u << (CANx_TWO_BYTE_OFFSET + \
                                                 CANx_RX_WPNH_SHIFT))))

#define CANx_RX_WPN_CLEAR             ((uint32) (~CANx_RX_WPN_SET))

#define CANx_TX_RSVD_MASK             ((uint32) 0x00FF00FFu)

#define CANx_TX_READ_BACK_MASK        (CANx_TX_WPN_CLEAR & CANx_TX_RSVD_MASK)

#define CANx_RX_READ_BACK_MASK        (CANx_RX_WPN_CLEAR & CANx_TX_RSVD_MASK)

#define CANx_RX_CMD_REG_WIDTH         (0x20u)

/* TX send message */
#define CANx_TX_REQUEST_PENDING     (0x01u)
#define CANx_RETRY_NUMBER           (0x03u)
#define CANx_SEND_MESSAGE_SHIFT     (0u)
#define CANx_SEND_MESSAGE           ((uint32) ((uint32) 0x00000001u << CANx_SEND_MESSAGE_SHIFT))

/* Offsets to maintain bytes within uint32 */
#define CANx_ONE_BYTE_OFFSET          (8u)
#define CANx_TWO_BYTE_OFFSET          (16u)
#define CANx_TREE_BYTE_OFFSET         (24u)

/* Set/Clear bits macro for CANx_RX maibox(i) */
/* bit 0 within RX_CMD[i] */
#define CANx_RX_ACK_MSG_SHIFT         (0u)
#define CANx_RX_ACK_MSG               ((uint8) ((uint8) 0x01u << CANx_RX_ACK_MSG_SHIFT))
#define CANx_RX_ACK_MESSAGE(i)        (CANx_RX[i].rxcmd.byte[0u] |= CANx_RX_ACK_MSG)
/* bit 2 within RX_CMD[i] */
#define CANx_RX_RTR_ABORT_SHIFT       (2u)
#define CANx_RX_RTR_ABORT_MASK        ((uint8) ((uint8) 0x01u << CANx_RX_RTR_ABORT_SHIFT))
#define CANx_RX_RTR_ABORT_MESSAGE(i)  (CANx_RX[i].rxcmd.byte[0u] |= \
                                                   CANx_RX_RTR_ABORT_MASK)
/* bit 3 within RX_CMD[i] */
#define CANx_RX_BUF_ENABLE_SHIFT      (3u)
#define CANx_RX_BUF_ENABLE_MASK       ((uint8) ((uint8) 0x01u << CANx_RX_BUF_ENABLE_SHIFT))
#define CANx_RX_BUF_ENABLE(i)         (CANx_RX[i].rxcmd.byte[0u] |= \
                                                   CANx_RX_BUF_ENABLE_MASK)
#define CANx_RX_BUF_DISABLE(i)        (CANx_RX[i].rxcmd.byte[0u] &= \
                                                   (uint8) (~CANx_RX_BUF_ENABLE_MASK))
/* bit 4 within RX_CMD[i] */
#define CANx_RX_RTRREPLY_SHIFT        (4u)
#define CANx_RX_RTRREPLY_MASK         ((uint8) ((uint8) 0x01u << CANx_RX_RTRREPLY_SHIFT))
#define CANx_SET_RX_RTRREPLY(i)       (CANx_RX[i].rxcmd.byte[0u] |= \
                                                   CANx_RX_RTRREPLY_MASK)
#define CANx_CLEAR_RX_RTRREPLY(i)     (CANx_RX[i].rxcmd.byte[0u] &= \
                                                   (uint8) (~CANx_RX_RTRREPLY_MASK))
/* bit 5 within RX_CMD[i] */
#define CANx_RX_INT_ENABLE_SHIFT      (5u)
#define CANx_RX_INT_ENABLE_MASK       ((uint8) ((uint8) 0x01u << CANx_RX_INT_ENABLE_SHIFT))
#define CANx_RX_INT_ENABLE(i)         (CANx_RX[i].rxcmd.byte[0u] |= \
                                                   CANx_RX_INT_ENABLE_MASK)
#define CANx_RX_INT_DISABLE(i)        (CANx_RX[i].rxcmd.byte[0u] &= \
                                                   (uint8) (~CANx_RX_INT_ENABLE_MASK))
/* bit 6 within RX_CMD[i] */
#define CANx_RX_LINKING_SHIFT         (6u)
#define CANx_RX_LINKING_MASK          ((uint8) ((uint8) 0x01u << CANx_RX_LINKING_SHIFT))
#define CANx_SET_RX_LINKING(i)        (CANx_RX[i].rxcmd.byte[0u] |= \
                                                   CANx_RX_LINKING_MASK)
#define CANx_CLEAR_RX_LINKING(i)      (CANx_RX[i].rxcmd.byte[0u] &= \
                                                   (uint8) (~CANx_RX_LINKING_MASK))
/* bit 7 within RX_CMD[i] */
#define CANx_RX_WPNL_SHIFT          (7u)
#define CANx_RX_WPNL_MASK           ((uint8) ((uint8) 0x01u << CANx_RX_WPNL_SHIFT))
#define CANx_SET_RX_WNPL(i)         (CANx_RX[i].rxcmd.byte[0u] |= CANx_RX_WPNL_MASK)
#define CANx_CLEAR_RX_WNPL(i)       (CANx_RX[i].rxcmd.byte[0u] &= \
                                                 (uint8) (~CANx_RX_WPNL_MASK))

/* bit 23 within RX_CMD[i] */
#define CANx_RX_WPNH_SHIFT          (7u)
#define CANx_RX_WPNH_MASK           ((uint8) ((uint8) 0x01u << CANx_RX_WPNH_SHIFT))
#define CANx_SET_RX_WNPH(i)         (CANx_RX[i].rxcmd.byte[2u] |= CANx_RX_WPNH_MASK)
#define CANx_CLEAR_RX_WNPH(i)       (CANx_RX[i].rxcmd.byte[2u] &= \
                                                 (uint8) (~CANx_RX_WPNH_MASK))

/* bits 19-16 within TX_CMD[i] */
#define CANx_RX_DLC_VALUE_SHIFT      (0u)
#define CANx_RX_DLC_VALUE_MASK       ((uint8) ((uint8) 0x0Fu << CANx_RX_DLC_VALUE_SHIFT))
#define CANx_GET_DLC(mailbox)        (CANx_RX[mailbox].rxcmd.byte[2u] & \
                                                  CANx_RX_DLC_VALUE_MASK)

/* Set/Clear bits macro for CANx_TX mailbox(i) */
/* bit 0 within TX_CMD[i] */
#define CANx_TX_TRANSMIT_REQUEST_SHIFT (0u)
#define CANx_TX_TRANSMIT_REQUEST     ((uint8) ((uint8) 0x01u << CANx_TX_TRANSMIT_REQUEST_SHIFT))
#define CANx_TX_TRANSMIT_MESSAGE(i)  (CANx_TX[i].txcmd.byte[0u] |= \
                                                  CANx_TX_TRANSMIT_REQUEST)
/* bit 1 within TX_CMD[i] */
#define CANx_TX_ABORT_SHIFT        (1u)
#define CANx_TX_ABORT_MASK         ((uint8) ((uint8) 0x01u << CANx_TX_ABORT_SHIFT))
#define CANx_TX_ABORT_MESSAGE(i)   (CANx_TX[i].txcmd.byte[0u] |= CANx_TX_ABORT_MASK)

/* bit 2 within TX_CMD[i] */
#define CANx_TRANSMIT_INT_ENABLE   (0x01u)
#define CANx_TRANSMIT_INT_DISABLE  (0x00u)
#define CANx_TX_INT_ENABLE_SHIFT   (2u)
#define CANx_TX_INT_ENABLE_MASK    ((uint32) ((uint32) 0x00000001u << CANx_TX_INT_ENABLE_SHIFT))
#define CANx_TX_INT_ENABLE(i)      (CANx_TX[i].txcmd.byte[0u] |= \
                                                (uint8) CANx_TX_INT_ENABLE_MASK)
#define CANx_TX_INT_DISABLE(i)     (CANx_TX[i].txcmd.byte[0u] &= \
                                                (uint8) (~CANx_TX_INT_ENABLE_MASK))
/* bit 3 within TX_CMD[i] */
#define CANx_TX_WPNL_SHIFT         (3u)
#define CANx_TX_WPNL_MASK          ((uint8) ((uint8) 0x01u << CANx_TX_WPNL_SHIFT))
#define CANx_SET_TX_WNPL(i)        (CANx_TX[i].txcmd.byte[0u] |= CANx_TX_WPNL_MASK)
#define CANx_CLEAR_TX_WNPL(i)      (CANx_TX[i].txcmd.byte[0u] &= \
                                                (uint8) (~CANx_TX_WPNL_MASK))

/* bits 19-16 within TX_CMD[i] */
#define CANx_TX_DLC_VALUE_SHIFT       (0u)
#define CANx_TX_DLC_UPPER_VALUE_SHIFT (19u)
#define CANx_TX_DLC_UPPER_VALUE       ((uint32) ((uint32) 0x00000001u << \
                                                   CANx_TX_DLC_UPPER_VALUE_SHIFT))
#define CANx_TX_DLC_VALUE_MASK        ((uint8) ((uint8) 0x0Fu << CANx_TX_DLC_VALUE_SHIFT))
#define CANx_TX_DLC_MAX_VALUE         (8u)

/* bit 20 within TX_CMD[i] */
#define CANx_TX_IDE_SHIFT             (20u)
#define CANx_TX_IDE_MASK              ((uint32) ((uint32) 0x00000001u << CANx_TX_IDE_SHIFT))
#define CANx_SET_TX_IDE(i)            (CANx_TX[i].txcmd.byte[2u] |= \
                                                   (uint8) (CANx_TX_IDE_MASK >> \
                                                   CANx_TWO_BYTE_OFFSET))
#define CANx_CLEAR_TX_IDE(i)          (CANx_TX[i].txcmd.byte[2u] &= \
                                                   (uint8) (((uint32) (~CANx_TX_IDE_MASK)) >> \
                                                   CANx_TWO_BYTE_OFFSET))

/* bit 21 within TX_CMD[i] */
#define CANx_TX_RTR_SHIFT             (21u)
#define CANx_TX_RTR_MASK              ((uint32) ((uint32) 0x00000001u <<  CANx_TX_RTR_SHIFT))
#define CANx_SET_TX_RTR(i)            (CANx_TX[i].txcmd.byte[2u] |= \
                                                   (uint8) (CANx_TX_RTR_MASK >> \
                                                   CANx_TWO_BYTE_OFFSET))
#define CANx_CLEAR_TX_RTR(i)          (CANx_TX[i].txcmd.byte[2u] &= \
                                                   (uint8) (((uint32) (~CANx_TX_RTR_MASK)) >> \
                                                   CANx_TWO_BYTE_OFFSET))

/* bit 23 within TX_CMD[i] */
#define CANx_TX_WPNH_SHIFT           (7u)
#define CANx_TX_WPNH_MASK            ((uint8) ((uint8) 0x01u << CANx_TX_WPNH_SHIFT))
#define CANx_SET_TX_WNPH(i)          (CANx_TX[i].txcmd.byte[2u] |= CANx_TX_WPN_MASK)
#define CANx_CLEAR_TX_WNPH(i)        (CANx_TX[i].txcmd.byte[2u] &= \
                                                  (uint8) (~CANx_TX_WPN_MASK))

/* bit 4 within RX_CMD[i].byte[2] */
#define CANx_RX_IDE_SHIFT             (4u)
#define CANx_RX_IDE_MASK              ((uint8) ((uint8) 0x01u << CANx_RX_IDE_SHIFT))
#define CANx_GET_RX_IDE(i)            (CANx_RX[i].rxcmd.byte[2u] & CANx_RX_IDE_MASK)
#define CANx_GET_RX_ID(i)             ((CANx_GET_RX_IDE(i)) ? ((CY_GET_REG32((reg32 *) & \
                                                    CANx_RX[i].rxid)) >> 3u) : ((CY_GET_REG32((reg32 *) & \
                                                    CANx_RX[i].rxid)) >> 21u))

#define CANx_RX_DATA_BYTE(mailbox, i) (CANx_RX[mailbox].rxdata.byte[((i) > 3u) ? \
                                                   (7u - ((i) - 4u)) : (3u - (i))])
#define CANx_TX_DATA_BYTE(mailbox, i) (CANx_TX[mailbox].txdata.byte[((i) > 3u) ? \
                                                   (7u - ((i) - 4u)) : (3u - (i))])

/* Macros for access to RX DATA for mailbox(i) */
#define CANx_RX_DATA_BYTE1(i)         CANx_RX[i].rxdata.byte[3u]
#define CANx_RX_DATA_BYTE2(i)         CANx_RX[i].rxdata.byte[2u]
#define CANx_RX_DATA_BYTE3(i)         CANx_RX[i].rxdata.byte[1u]
#define CANx_RX_DATA_BYTE4(i)         CANx_RX[i].rxdata.byte[0u]
#define CANx_RX_DATA_BYTE5(i)         CANx_RX[i].rxdata.byte[7u]
#define CANx_RX_DATA_BYTE6(i)         CANx_RX[i].rxdata.byte[6u]
#define CANx_RX_DATA_BYTE7(i)         CANx_RX[i].rxdata.byte[5u]
#define CANx_RX_DATA_BYTE8(i)         CANx_RX[i].rxdata.byte[4u]

/* Macros for access to TX DATA for mailbox(i) */
#define CANx_TX_DATA_BYTE1(i)         CANx_TX[i].txdata.byte[3u]
#define CANx_TX_DATA_BYTE2(i)         CANx_TX[i].txdata.byte[2u]
#define CANx_TX_DATA_BYTE3(i)         CANx_TX[i].txdata.byte[1u]
#define CANx_TX_DATA_BYTE4(i)         CANx_TX[i].txdata.byte[0u]
#define CANx_TX_DATA_BYTE5(i)         CANx_TX[i].txdata.byte[7u]
#define CANx_TX_DATA_BYTE6(i)         CANx_TX[i].txdata.byte[6u]
#define CANx_TX_DATA_BYTE7(i)         CANx_TX[i].txdata.byte[5u]
#define CANx_TX_DATA_BYTE8(i)         CANx_TX[i].txdata.byte[4u]

/* Macros for set Tx Msg Indentifier in CANx_TX_ID register */
#define CANx_SET_TX_ID_STANDARD_MSG_SHIFT  (21u)
#define CANx_SET_TX_ID_EXTENDED_MSG_SHIFT  (3u)
#define CANx_SET_TX_ID_STANDARD_MSG(i, id) (CY_SET_REG32((reg32 *) &CANx_TX[i].txid, \
                                        ((uint32) ((uint32) (id) << CANx_SET_TX_ID_STANDARD_MSG_SHIFT))))
#define CANx_SET_TX_ID_EXTENDED_MSG(i, id) (CY_SET_REG32((reg32 *) &CANx_TX[i].txid, \
                                        ((uint32) ((uint32) (id) << CANx_SET_TX_ID_EXTENDED_MSG_SHIFT))))

/* Mask for bits within CANx_CSR_CFG */
#define CANx_EDGE_MODE_SHIFT               (0u)
/* bit 0 within CSR_CFG */
#define CANx_EDGE_MODE_MASK                ((uint8) ((uint8) 0x01u << CANx_EDGE_MODE_SHIFT))
#define CANx_SAMPLE_MODE_SHIFT             (1u)
/* bit 1 within CSR_CFG */
#define CANx_SAMPLE_MODE_MASK              ((uint8) ((uint8) 0x01u << CANx_SAMPLE_MODE_SHIFT))
#define CANx_CFG_REG_SJW_SHIFT             (2u)
/* bits 3-2 within CSR_CFG */
#define CANx_CFG_REG_SJW_MASK              ((uint8) ((uint8) 0x03u << CANx_CFG_REG_SJW_SHIFT))
#define CANx_CFG_REG_SJW_LOWER_LIMIT       (0x03u)  /* the lowest allowed value of cfg_sjw */
#define CANx_RESET_SHIFT                   (4u)
/* bit 4 within CSR_CFG    */
#define CANx_RESET_MASK                    ((uint8) ((uint8) 0x01u << CANx_RESET_SHIFT))
#define CANx_CFG_REG_TSEG2_SHIFT           (5u)
/* bits 7-5 within CSR_CFG */
#define CANx_CFG_REG_TSEG2_MASK            ((uint8) ((uint8) 0x07u << CANx_CFG_REG_TSEG2_SHIFT))
/* the highest allowed value of cfg_tseg2 */
#define CANx_CFG_REG_TSEG2_UPPER_LIMIT     (0x07u)
/* the lowest allowed value of cfg_tseg2 */
#define CANx_CFG_REG_TSEG2_LOWER_LIMIT     (0x02u)
 /* the lowest allowed value of cfg_tseg2 if sample point is one point */
#define CANx_CFG_REG_TSEG2_EXCEPTION       (0x01u)  
/* bits 11-8 within CSR_CFG */
#define CANx_CFG_REG_TSEG1_SHIFT           (8u)
#define CANx_CFG_REG_TSEG1_MASK            (0x0Fu)
/* the highest allowed value of cfg_tseg1 */
#define CANx_CFG_REG_TSEG1_UPPER_LIMIT     (0x0Fu)
/* the lowest allowed value of cfg_tseg1 */
#define CANx_CFG_REG_TSEG1_LOWER_LIMIT     (0x02u)
#define CANx_ARBITRATION_SHIFT             (4u)
#define CANx_ARBITER_SHIFT                 (12u)
/* bit 12 within CSR_CFG */
#define CANx_ARBITRATION_MASK              ((uint8) ((uint8) 0x01u << CANx_ARBITRATION_SHIFT))
/* bits 23-16 within CSR_CFG */
#define CANx_BITRATE_SHIFT                 (16u)
#define CANx_BITRATE_MASK                  (0x7FFFu)

/* Mask for bits within CANx_CSR_CMD */
#define CANx_MODE_SHIFT                    (0u)
/* bit 0 within CSR_CMD */
#define CANx_MODE_MASK                     ((uint8) ((uint8) 0x01u << CANx_MODE_SHIFT))
#define CANx_OPMODE_MASK_SHIFT             (1u)
/* bit 1 within CSR_CMD */
#define CANx_OPMODE_MASK                   ((uint8) ((uint8) 0x01u << CANx_OPMODE_MASK_SHIFT))

/* Mask for bits within CANx_CSR_CMD */
#define CANx_ERROR_STATE_SHIFT             (0u)
/* bit 17-16 within ERR_SR */
#define CANx_ERROR_STATE_MASK              ((uint8) ((uint8) 0x03u << CANx_ERROR_STATE_SHIFT))
#define CANx_TX_ERROR_FLAG_SHIFT           (2u)
/* bit 18 within ERR_SR */
#define CANx_TX_ERROR_FLAG_MASK            ((uint8) ((uint8) 0x01u << CANx_TX_ERROR_FLAG_SHIFT))
#define CANx_RX_ERROR_FLAG_SHIFT           (3u)
/* bit 19 within ERR_SR */
#define CANx_RX_ERROR_FLAG_MASK            ((uint8) ((uint8) 0x01u << CANx_RX_ERROR_FLAG_SHIFT))

/* Mask and Macros for bits within CANx_INT_EN_REG */
#define CANx_GLOBAL_INT_SHIFT              (0u)
/* bit 0 within INT_EN */
#define CANx_GLOBAL_INT_MASK               ((uint8) ((uint8) 0x01u << CANx_GLOBAL_INT_SHIFT))
#define CANx_ARBITRATION_LOST_SHIFT        (2u)
/* bit 2 within INT_EN and INT_SR */
#define CANx_ARBITRATION_LOST_MASK      ((uint8) ((uint8) 0x01u << CANx_ARBITRATION_LOST_SHIFT))
#define CANx_ARBITRATION_LOST_INT_ENABLE   (CANx_INT_EN_REG.byte[0u] |= \
                                                        CANx_ARBITRATION_LOST_MASK)
#define CANx_ARBITRATION_LOST_INT_DISABLE  (CANx_INT_EN_REG.byte[0u] &= \
                                                       (uint8) (~CANx_ARBITRATION_LOST_MASK))
#define CANx_OVERLOAD_ERROR_SHIFT          (3u)
/* bit 3 within INT_EN and INT_SR */
#define CANx_OVERLOAD_ERROR_MASK          ((uint8) ((uint8) 0x01u << CANx_OVERLOAD_ERROR_SHIFT))
#define CANx_OVERLOAD_ERROR_INT_ENABLE    (CANx_INT_EN_REG.byte[0u] |= \
                                                       CANx_OVERLOAD_ERROR_MASK)
#define CANx_OVERLOAD_ERROR_INT_DISABLE   (CANx_INT_EN_REG.byte[0u] &= \
                                                      (uint8) (~CANx_OVERLOAD_ERROR_MASK))
#define CANx_BIT_ERROR_SHIFT               (4u)
/* bit 4 within INT_EN and INT_SR */
#define CANx_BIT_ERROR_MASK                ((uint8) ((uint8) 0x01u << CANx_BIT_ERROR_SHIFT))
#define CANx_BIT_ERROR_LOST_INT_ENABLE     (CANx_INT_EN_REG.byte[0u] |= \
                                                        CANx_BIT_ERROR_MASK)
#define CANx_BIT_ERROR_LOST_INT_DISABLE    (CANx_INT_EN_REG.byte[0u] &= \
                                                       (uint8) (~CANx_BIT_ERROR_MASK))
#define CANx_STUFF_ERROR_SHIFT             (5u)
/* bit 5 within INT_EN and INT_SR */
#define CANx_STUFF_ERROR_MASK              ((uint8) ((uint8) 0x01u << CANx_STUFF_ERROR_SHIFT))
#define CANx_STUFF_ERROR_INT_ENABLE        (CANx_INT_EN_REG.byte[0u] |= \
                                                        CANx_STUFF_ERROR_MASK)
#define CANx_STUFF_ERROR_INT_DISABLE       (CANx_INT_EN_REG.byte[0u] &= \
                                                       (uint8) (~CANx_STUFF_ERROR_MASK))
#define CANx_ACK_ERROR_SHIFT               (6u)
/* bit 6 within INT_EN and INT_SR */
#define CANx_ACK_ERROR_MASK                ((uint8) ((uint8) 0x01u << CANx_ACK_ERROR_SHIFT))
#define CANx_ACK_ERROR_INT_ENABLE          (CANx_INT_EN_REG.byte[0u] |= \
                                                        CANx_ACK_ERROR_MASK)
#define CANx_ACK_ERROR_INT_DISABLE         (CANx_INT_EN_REG.byte[0u] &= \
                                                       (uint8) (~CANx_ACK_ERROR_MASK))
#define CANx_FORM_ERROR_SHIFT              (7u)
/* bit 7 within INT_EN and INT_SR */
#define CANx_FORM_ERROR_MASK               ((uint8) ((uint8) 0x01u << CANx_FORM_ERROR_SHIFT))
#define CANx_FORM_ERROR_INT_ENABLE         (CANx_INT_EN_REG.byte[0u] |= \
                                                        CANx_FORM_ERROR_MASK)
#define CANx_FORM_ERROR_INT_DISABLE        (CANx_INT_EN_REG.byte[0u] &= \
                                                        (uint8) (~CANx_FORM_ERROR_MASK))
#define CANx_CRC_ERROR_SHIFT               (0u)
/* bit 8 within INT_EN and INT_SR */
#define CANx_CRC_ERROR_MASK                ((uint8) ((uint8) 0x01u << CANx_CRC_ERROR_SHIFT))
#define CANx_CRC_ERROR_INT_ENABLE          (CANx_INT_EN_REG.byte[1u] |= \
                                                        CANx_CRC_ERROR_MASK)
#define CANx_CRC_ERROR_INT_DISABLE         (CANx_INT_EN_REG.byte[1u] &= \
                                                        (uint8) (~CANx_CRC_ERROR_MASK))
#define CANx_BUS_OFF_SHIFT                 (1u)
/* bit 9 within INT_EN and INT_SR */
#define CANx_BUS_OFF_MASK                  ((uint8) ((uint8) 0x01u << CANx_BUS_OFF_SHIFT))
#define CANx_BUS_OFF_INT_ENABLE            (CANx_INT_EN_REG.byte[1u] |= \
                                                        CANx_BUS_OFF_MASK)
#define CANx_BUS_OFF_INT_DISABLE           (CANx_INT_EN_REG.byte[1u] &= \
                                                        (uint8) (~CANx_BUS_OFF_MASK))
#define CANx_RX_MSG_LOST_SHIFT             (2u)
/* bit 10 within INT_EN and INT_SR */
#define CANx_RX_MSG_LOST_MASK              ((uint8) ((uint8) 0x01u << CANx_RX_MSG_LOST_SHIFT))
#define CANx_RX_MSG_LOST_INT_ENABLE        (CANx_INT_EN_REG.byte[1u] |= \
                                                        CANx_RX_MSG_LOST_MASK)
#define CANx_RX_MSG_LOST_INT_DISABLE       (CANx_INT_EN_REG.byte[1u] &= \
                                                        (uint8) (~CANx_RX_MSG_LOST_MASK))
#define CANx_TX_MESSAGE_SHIFT              (3u)
/* bit 11 within INT_EN and INT_SR */
#define CANx_TX_MESSAGE_MASK               ((uint8) ((uint8) 0x01u << CANx_TX_MESSAGE_SHIFT))
#define CANx_TX_MSG_INT_ENABLE             (CANx_INT_EN_REG.byte[1u] |= \
                                                        CANx_TX_MESSAGE_MASK)
#define CANx_TX_MSG_INT_DISABLE            (CANx_INT_EN_REG.byte[1u] &= \
                                                        (uint8) (~CANx_TX_MESSAGE_MASK))
#define CANx_RX_MESSAGE_SHIFT              (4u)
/* bit 12 within INT_EN and INT_SR */
#define CANx_RX_MESSAGE_MASK               ((uint8) ((uint8) 0x01u << CANx_RX_MESSAGE_SHIFT))
#define CANx_RX_MSG_INT_ENABLE             (CANx_INT_EN_REG.byte[1u] |= \
                                                        CANx_RX_MESSAGE_MASK)
#define CANx_RX_MSG_INT_DISABLE            (CANx_INT_EN_REG.byte[1u] &= \
                                                        (uint8) (~CANx_RX_MESSAGE_MASK))

#define CANx_GLOBAL_INT_ENABLE_SHIFT       (0u)
#define CANx_ARBITRATION_LOST_ENABLE_SHIFT (2u)
#define CANx_OVERLOAD_ERROR_ENABLE_SHIFT   (3u)
#define CANx_BIT_ERROR_ENABLE_SHIFT        (4u)
#define CANx_STUFF_ERROR_ENABLE_SHIFT      (5u)
#define CANx_ACK_ERROR_ENABLE_SHIFT        (6u)
#define CANx_FORM_ERROR_ENABLE_SHIFT       (7u)
#define CANx_CRC_ERROR_ENABLE_SHIFT        (8u)
#define CANx_BUS_OFF_ENABLE_SHIFT          (9u)
#define CANx_RX_MSG_LOST_ENABLE_SHIFT      (10u)
#define CANx_TX_MESSAGE_ENABLE_SHIFT       (11u)
#define CANx_RX_MESSAGE_ENABLE_SHIFT       (12u)

#define CANx_GLOBAL_INT_ENABLE       ((uint16) ((uint16) 0x01u << CANx_GLOBAL_INT_ENABLE_SHIFT))
#define CANx_ARBITRATION_LOST_ENABLE ((uint16) ((uint16) 0x01u << \
                                                            CANx_ARBITRATION_LOST_ENABLE_SHIFT))
#define CANx_OVERLOAD_ERROR_ENABLE   ((uint16) ((uint16) 0x01u << \
                                                            CANx_OVERLOAD_ERROR_ENABLE_SHIFT))
#define CANx_BIT_ERROR_ENABLE       ((uint16) ((uint16) 0x01u << CANx_BIT_ERROR_ENABLE_SHIFT))
#define CANx_STUFF_ERROR_ENABLE     ((uint16) ((uint16) 0x01u << CANx_STUFF_ERROR_ENABLE_SHIFT))
#define CANx_ACK_ERROR_ENABLE       ((uint16) ((uint16) 0x01u << CANx_ACK_ERROR_ENABLE_SHIFT))
#define CANx_FORM_ERROR_ENABLE      ((uint16) ((uint16) 0x01u << CANx_FORM_ERROR_ENABLE_SHIFT))
#define CANx_CRC_ERROR_ENABLE       ((uint16) ((uint16) 0x01u << CANx_CRC_ERROR_ENABLE_SHIFT))
#define CANx_BUS_OFF_ENABLE         ((uint16) ((uint16) 0x01u << CANx_BUS_OFF_ENABLE_SHIFT))
#define CANx_RX_MSG_LOST_ENABLE     ((uint16) ((uint16) 0x01u << CANx_RX_MSG_LOST_ENABLE_SHIFT))
#define CANx_TX_MESSAGE_ENABLE      ((uint16) ((uint16) 0x01u << CANx_TX_MESSAGE_ENABLE_SHIFT))
#define CANx_RX_MESSAGE_ENABLE      ((uint16) ((uint16) 0x01u << CANx_RX_MESSAGE_ENABLE_SHIFT))

/******************************************************************************************
* Following code are OBSOLETE and must not be used starting from CAN 2.30
******************************************************************************************/
#define CANx_RESTART_BY_HAND               (0x00u)

#endif /* CY_CAN_CANx_H */


/* [] END OF FILE */
