/*******************************************************************************
* File Name: CANx_INT.c
* Version 2.30
*
* Description:
*  This file contains the Interrupt Service Routine (ISR) for the CAN
*  Component. Interrupt handlers functions generate accordingly to PSoC Creator 
*  Customizer inputs.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "CANx.h"

/* `#START CAN_INT_C_CODE_DEFINITION` */

/* `#END` */

#if (CANx_ARB_LOST)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_ArbLost
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Arbitration Lost Interrupt. Clears
    *  Arbitration Lost interrupt flag. Only generated if Arbitration Lost
    *  Interrupt enable in Customizer.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    * 
    * Reentrant:
    *  Depends on Customer code.
    *
    *******************************************************************************/
    void CANx_ArbLostIsr(void) 
    {
        /* Clear Arbitration Lost flag */
        CANx_INT_SR_REG.byte[0u] = CANx_ARBITRATION_LOST_MASK;
    
        /* `#START ARBITRATION_LOST_ISR` */

        /* `#END` */
    }
    
#endif /* CANx_ARB_LOST */

#if (CANx_OVERLOAD)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_OvrLdErrror
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Overload Error Interrupt. Clears Overload
    *  Error interrupt flag. Only generated if Overload Error Interrupt enable
    *  in Customizer.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.        
    *
    * Reentrant:
    *  Depends on Customer code.
    *
    *******************************************************************************/
    void CANx_OvrLdErrorIsr(void) 
    {
        /* Clear Overload Error flag */
        CANx_INT_SR_REG.byte[0u] = CANx_OVERLOAD_ERROR_MASK;
    
        /* `#START OVER_LOAD_ERROR_ISR` */

        /* `#END` */
    }
    
#endif /* CANx_OVERLOAD */

#if (CANx_BIT_ERR)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_BitError
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Bit Error Interrupt. Clears Bit Error
    *  interrupt flag. Only generated if Bit Error Interrupt enable in Customizer.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Reentrant:
    *  Depends on Customer code.
    *
    *******************************************************************************/
    void CANx_BitErrorIsr(void) 
    {
        /* Clear Bit Error flag */
        CANx_INT_SR_REG.byte[0u] = CANx_BIT_ERROR_MASK;
    
        /* `#START BIT_ERROR_ISR` */

        /* `#END` */
    }
    
#endif /* CANx_BIT_ERR */

#if (CANx_STUFF_ERR)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_BitStuffError
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Bit Stuff Error Interrupt. Clears Bit Stuff
    *  Error interrupt flag. Only generated if Bit Stuff Error Interrupt enable
    *  in Customizer.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Reentrant:
    *  Depends on Customer code.
    *
    *******************************************************************************/
    void CANx_BitStuffErrorIsr(void) 
    {
        /* Clear Stuff Error flag */
        CANx_INT_SR_REG.byte[0u] = CANx_STUFF_ERROR_MASK;
    
        /* `#START BIT_STUFF_ERROR_ISR` */

        /* `#END` */
    }
    
#endif /* CANx_STUFF_ERR */

#if (CANx_ACK_ERR)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_AckErrorIsr
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Acknowledge Error Interrupt. Clears
    *  Acknowledge Error interrupt flag. Only generated if Acknowledge Error
    *  Interrupt enable in Customizer.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Reentrant:
    *  Depends on Customer code.
    *
    *******************************************************************************/
    void CANx_AckErrorIsr(void) 
    {
        /* Clear Acknoledge Error flag */
        CANx_INT_SR_REG.byte[0u] = CANx_ACK_ERROR_MASK;
    
        /* `#START ACKNOWLEDGE_ERROR_ISR` */

        /* `#END` */
    }
    
#endif /* CANx_ACK_ERR */

#if (CANx_FORM_ERR)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_MsgError
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Form Error Interrupt. Clears Form Error
    *  interrupt flag. Only generated if Form Error Interrupt enable in Customizer.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Reentrant:
    *  Depends on Customer code.
    *
    *******************************************************************************/
    void CANx_MsgErrorIsr(void) 
    {
        /* Clear Form Error flag */
        CANx_INT_SR_REG.byte[0u] = CANx_FORM_ERROR_MASK;
    
        /* `#START MESSAGE_ERROR_ISR` */

        /* `#END` */
    }
    
#endif /* CANx_FORM_ERR */

#if (CANx_CRC_ERR)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_CrcError
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to CRC Error Interrupt. Clears CRC Error
    *  interrupt flag. Only generated if CRC Error Interrupt enable in Customizer.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Reentrant:
    *  Depends on Customer code.
    *
    *******************************************************************************/
    void CANx_CrcErrorIsr(void) 
    {
        /* Clear CRC Error flag */
        CANx_INT_SR_REG.byte[1u] = CANx_CRC_ERROR_MASK;
    
        /* `#START CRC_ERROR_ISR` */

        /* `#END` */
    }
    
#endif /* CANx_CRC_ERR */

#if (CANx_BUS_OFF)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_BusOff
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Bus Off Interrupt. Places CAN Component
    *  to Stop mode. Only generated if Bus Off Interrupt enable in Customizer.
    *  Recommended setting to enable this interrupt.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Side Effects:
    *  Disable power to the CAN Core.
    *
    *******************************************************************************/
    void CANx_BusOffIsr(void) 
    {        
        /* Clear Bus Off flag */
        CANx_INT_SR_REG.byte[1u] = CANx_BUS_OFF_MASK;
    
        /* `#START BUS_OFF_ISR` */

        /* `#END` */
        (void) CANx_Stop();
    }
    
#endif /* CANx_BUS_OFF */

#if (CANx_RX_MSG_LOST)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_MsgLost
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Message Lost Interrupt. Clears Message Lost
    *  interrupt flag. Only generated if Message Lost Interrupt enable in Customizer
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Reentrant:
    *  Depends on Customer code.
    *
    *******************************************************************************/
    void CANx_MsgLostIsr(void) 
    {
        /* Clear Receive Message Lost flag */
        CANx_INT_SR_REG.byte[1u] = CANx_RX_MSG_LOST_MASK;
    
        /* `#START MESSAGE_LOST_ISR` */

        /* `#END` */
    }
    
#endif /* CANx_RX_MSG_LOST */

#if (CANx_TX_MESSAGE)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_MsgTXIsr
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Transmit Message Interrupt. Clears Transmit
    *  Message interrupt flag. Only generated if Transmit Message Interrupt enable
    *  in Customizer.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.    
    *
    * Reentrant:
    *  Depends on Customer code.
    *
    *******************************************************************************/
    void CANx_MsgTXIsr(void) 
    {
        /* Clear Transmit Message flag */
        CANx_INT_SR_REG.byte[1u] = CANx_TX_MESSAGE_MASK;
        
        /* `#START MESSAGE_TRANSMITTED_ISR` */

        /* `#END` */
    }
    
#endif /* CANx_TX_MESSAGE */

#if (CANx_RX_MESSAGE)

    /*******************************************************************************
    * FUNCTION NAME:   CANx_MsgRXIsr
    ********************************************************************************
    *
    * Summary:
    *  This function is entry point to Receive Message Interrupt. Clears Receive
    *  Message interrupt flag and call appropriate handlers for Basic and Full
    *  interrupt based mailboxes. Only generated if Receive Message Interrupt
    *  enable in Customizer. Recommended setting to enable this interrupt.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void CANx_MsgRXIsr(void) 
    {
        uint8 i;
        uint16 shift = 0x01u;
    
        /* Clear Receive Message flag */
        CANx_INT_SR_REG.byte[1u] = CANx_RX_MESSAGE_MASK;
        
        /* `#START MESSAGE_RECEIVE_ISR` */

        /* `#END` */
    
        for (i = 0u; i < CANx_NUMBER_OF_RX_MAILBOXES; i++)
        {
            if ((CY_GET_REG16((reg16 *) &CANx_BUF_SR_REG.byte[0u]) & shift) == shift)
            {
                if ((CANx_RX[i].rxcmd.byte[0u] & CANx_RX_INT_ENABLE_MASK) ==
                    CANx_RX_INT_ENABLE_MASK)
                {
                    if ((CANx_RX_MAILBOX_TYPE & shift) == shift)
                    {
                        /* RX Full mailboxes handler */
                        switch(i)
                        {
							case 0u : CANx_ReceiveMsg0();
							break;
							case 1u : CANx_ReceiveMsg1();
							break;
							case 2u : CANx_ReceiveMsgBoot_cmd();
							break;
                            default:
                            break;
                        }
                    }
                    else
                    {
                        /* RX Basic mailbox handler */
                        CANx_ReceiveMsg(i);
                    }
                }
            }
            shift <<= 1u;
        }            
    }
    
#endif /* CANx_RX_MESSAGE */


/*******************************************************************************
* Function Name: CANx_ISR
********************************************************************************
*
* Summary:
*  This ISR is executed when CAN core generate interrupt on one of events:
*  Arb_lost, Overload, Bit_err, Stuff_err, Ack_err, Form_err, Crc_err,
*  Buss_off, Rx_msg_lost, Tx_msg or Rx_msg. The interrupt sources depends
*  on Customizer inputs.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
CY_ISR(CANx_ISR)
{
    /* Place your Interrupt code here. */
    /* `#START CAN_ISR` */

    /* `#END` */    
    
    /* Arbitration */
    #if (CANx_ARB_LOST && (CANx_ARB_LOST_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[0u] & CANx_ARBITRATION_LOST_MASK) ==
             CANx_ARBITRATION_LOST_MASK)
        {
            CANx_ArbLostIsr();
        } 
    #endif /* CANx_ARB_LOST && CANx_ARB_LOST_USE_HELPER */
    
    /* Overload Error */
    #if (CANx_OVERLOAD && (CANx_OVERLOAD_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[0u] & CANx_OVERLOAD_ERROR_MASK) ==
             CANx_OVERLOAD_ERROR_MASK)
        {
            CANx_OvrLdErrorIsr();
        }
    #endif /* CANx_OVERLOAD && CANx_OVERLOAD_USE_HELPER */
    
    /* Bit Error */
    #if (CANx_BIT_ERR && (CANx_BIT_ERR_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[0u] & CANx_BIT_ERROR_MASK) == CANx_BIT_ERROR_MASK)
        {
            CANx_BitErrorIsr();
        }
    #endif /* CANx_BIT_ERR && CANx_BIT_ERR_USE_HELPER */
    
    /* Bit Staff Error */
    #if (CANx_STUFF_ERR && (CANx_STUFF_ERR_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[0u] & CANx_STUFF_ERROR_MASK) ==
             CANx_STUFF_ERROR_MASK)
        {
            CANx_BitStuffErrorIsr();
        }
    #endif /* CANx_STUFF_ERR && CANx_STUFF_ERR_USE_HELPER */
    
    /* ACK Error */
    #if (CANx_ACK_ERR && (CANx_ACK_ERR_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[0u] & CANx_ACK_ERROR_MASK) ==
             CANx_ACK_ERROR_MASK)
        {
            CANx_AckErrorIsr();
        }
    #endif /* CANx_ACK_ERR && CANx_ACK_ERR_USE_HELPER */
    
    /* Form(msg) Error */
    #if (CANx_FORM_ERR && (CANx_FORM_ERR_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[0u] & CANx_FORM_ERROR_MASK) ==
             CANx_FORM_ERROR_MASK)
        {
            CANx_MsgErrorIsr();
        }
    #endif /* CANx_FORM_ERR && CANx_FORM_ERR_USE_HELPER */
    
    /* CRC Error */
    #if (CANx_CRC_ERR && (CANx_CRC_ERR_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[1u] & CANx_CRC_ERROR_MASK) == CANx_CRC_ERROR_MASK)
        {
            CANx_CrcErrorIsr();
        }
    #endif /* CANx_CRC_ERR && CANx_CRC_ERR_USE_HELPER */
    
    /* Bus Off state */
    #if (CANx_BUS_OFF && (CANx_BUS_OFF_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[1u] & CANx_BUS_OFF_MASK) == CANx_BUS_OFF_MASK)
        {
            CANx_BusOffIsr();
        }
    #endif /* CANx_BUS_OFF && CANx_BUS_OFF_USE_HELPER */
    
    /* Message Lost */
    #if (CANx_RX_MSG_LOST && (CANx_RX_MSG_LOST_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[1u] & CANx_RX_MSG_LOST_MASK) ==
             CANx_RX_MSG_LOST_MASK)
        {
            CANx_MsgLostIsr();
        }
    #endif /* CANx_RX_MSG_LOST && CANx_RX_MSG_LOST_USE_HELPER */
    
    /* TX Message Send */
    #if (CANx_TX_MESSAGE && (CANx_TX_MESSAGE_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[1u] & CANx_TX_MESSAGE_MASK) ==
             CANx_TX_MESSAGE_MASK)
        {
            CANx_MsgTXIsr();
        }
    #endif /* CANx_TX_MESSAGE && CANx_TX_MESSAGE_USE_HELPER */
    
    /* RX Message Available */
    #if (CANx_RX_MESSAGE && (CANx_RX_MESSAGE_USE_HELPER || \
        (!CANx_ADVANCED_INTERRUPT_CFG)))
        if ((CANx_INT_SR_REG.byte[1u] & CANx_RX_MESSAGE_MASK) ==
             CANx_RX_MESSAGE_MASK)
        {
            CANx_MsgRXIsr();
        }
    #endif /* CANx_RX_MESSAGE && CANx_RX_MESSAGE_USE_HELPER */


}


/* [] END OF FILE */
